@@ -715,9 +715,9 @@ GpgolRibbonExtender::GetIDsOfNames (REFIID riid, LPOLESTR *rgszNames,
       ID_MAPPER (L"launchDetails", ID_LAUNCH_CERT_DETAILS)
       ID_MAPPER (L"getIsDetailsEnabled", ID_GET_IS_DETAILS_ENABLED)
       ID_MAPPER (L"getIsCrypto", ID_GET_IS_CRYPTO_MAIL)
-      ID_MAPPER (L"printDecrypted", ID_CMD_PRINT_DECRYPTED)
       ID_MAPPER (L"openContactKey", ID_CMD_OPEN_CONTACT_KEY)
       ID_MAPPER (L"overrideFileClose", ID_CMD_FILE_CLOSE)
+      ID_MAPPER (L"overrideFileSaveAs", ID_CMD_FILE_SAVE_AS)
       ID_MAPPER (L"decryptPermanently", ID_CMD_DECRYPT_PERMANENTLY)
     }
 
@@ -735,8 +735,13 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
                              EXCEPINFO *exepinfo, UINT *argerr)
 {
   USE_INVOKE_ARGS
-  log_debug ("%s:%s: enter with dispid: %x",
-             SRCNAME, __func__, (int)dispid);
+  log_oom ("%s:%s: enter with dispid: %x",
+           SRCNAME, __func__, (int)dispid);
+
+  /*
+  log_oom ("%s:%s: Parms: %s",
+           SRCNAME, __func__, format_dispparams (parms).c_str ());
+  */
 
   if (!(flags & DISPATCH_METHOD))
     {
@@ -804,8 +809,6 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
             options_dialog_box (NULL);
             return S_OK;
           }
-      case ID_CMD_PRINT_DECRYPTED:
-        return print_decrypted (parms->rgvarg[0].pdispVal);
       case ID_CMD_DECRYPT_PERMANENTLY:
         return decrypt_permanently (parms->rgvarg[0].pdispVal);
       case ID_GET_IS_CRYPTO_MAIL:
@@ -814,6 +817,8 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
         return open_contact_key (parms->rgvarg[0].pdispVal);
       case ID_CMD_FILE_CLOSE :
         return override_file_close ();
+      case ID_CMD_FILE_SAVE_AS:
+        return override_file_save_as (parms);
       case ID_BTN_ENCRYPT:
       case ID_BTN_DECRYPT:
       case ID_BTN_DECRYPT_LARGE:
@@ -957,14 +962,14 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         optsSTip
         );
     }
-  /* We don't use this code currently because calling the send
-     event for Inline Response mailitems fails. */
   else if (!wcscmp (RibbonID, L"Microsoft.Outlook.Explorer") && g_ol_version_major > 14)
     {
       gpgrt_asprintf (&buffer,
         "<customUI xmlns=\"http://schemas.microsoft.com/office/2009/07/customui\""
         " onLoad=\"ribbonLoaded\">"
         " <commands>"
+        "  <command idMso=\"FileSaveAs\""
+        "           onAction=\"overrideFileSaveAs\"/>"
         "  <command idMso=\"FileCloseAndLogOff\""
         "           onAction=\"overrideFileClose\"/>"
         " </commands>"
@@ -1030,14 +1035,6 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         " </ribbon>"
         " <contextMenus>"
         "  <contextMenu idMso=\"ContextMenuMailItem\">"
-        "   <button id=\"decryptedPrintBtn\""
-//        "idMso=\"FilePrintQuick\""
-        "           label=\"%s\""
-        "           onAction=\"printDecrypted\""
-        "           getImage=\"btnEncryptSmall\""
-        "           getVisible=\"getIsCrypto\""
-        "           insertAfterMso=\"FilePrintQuick\""
-        "   />"
         "   <button id=\"decryptPermanentlyBtn\""
         "           label=\"%s\""
         "           onAction=\"decryptPermanently\""
@@ -1055,7 +1052,6 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
         _("Sign"), signTTip, signSTip,
         _("Encrypt"), encryptTTip, encryptSTip,
         optsSTip,
-        _("&amp;Print decrypted"),
         _("Permanently &amp;decrypt")
         );
     }
