@@ -49,6 +49,7 @@ const wchar_t *prop_blacklist[] = {
   L"InternetCodepage",
   L"ConversationIndex",
   L"Subject",
+  L"SentOnBehalfOfName",
   NULL };
 
 typedef enum
@@ -144,6 +145,37 @@ EVENT_SINK_INVOKE(MailItemEvents)
   bool is_reply = false;
   switch(dispid)
     {
+      case BeforeAutoSave:
+        {
+          log_oom ("%s:%s: BeforeAutoSave : %p",
+                   SRCNAME, __func__, m_mail);
+          if (parms->cArgs != 1 || parms->rgvarg[0].vt != (VT_BOOL | VT_BYREF))
+           {
+             /* This happens in the weird case */
+             log_debug ("%s:%s: Uncancellable BeforeAutoSave.",
+                        SRCNAME, __func__);
+             TBREAK;
+           }
+
+          if (m_mail->isCryptoMail() && !m_mail->decryptedSuccessfully ())
+            {
+              *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+              log_debug ("%s:%s: Autosave for not successfuly decrypted mail."
+                         "Cancel it.",
+                         SRCNAME, __func__);
+              TBREAK;
+            }
+
+          if (opt.draft_key && (m_mail->needs_crypto_m () & 1) &&
+              !m_mail->isDraftEncrypt())
+            {
+              log_debug ("%s:%s: Draft encryption for autosave starting now.",
+                         SRCNAME, __func__);
+              m_mail->setIsDraftEncrypt (true);
+              m_mail->prepareCrypto_o ();
+            }
+          TRETURN S_OK;
+        }
       case Open:
         {
           log_oom ("%s:%s: Open : %p",
@@ -188,6 +220,13 @@ EVENT_SINK_INVOKE(MailItemEvents)
               log_error ("%s:%s: Pre process message failed.",
                          SRCNAME, __func__);
             }
+
+          if (m_mail->isDecryptAgain ())
+            {
+              log_debug ("%s:%s: Decrypting after 500ms",
+                         SRCNAME, __func__);
+              do_in_ui_thread_async (DECRYPT, m_mail, 500);
+            }
           TBREAK;
         }
       case Read:
@@ -275,13 +314,13 @@ EVENT_SINK_INVOKE(MailItemEvents)
             {
               if (!wcscmp (prop_name, *cur))
                 {
-                  log_oom ("%s:%s: Message %p propchange: %ls discarded.",
-                           SRCNAME, __func__, m_object, prop_name);
+                  log_oom ("%s:%s: Mail %p propchange: %ls discarded.",
+                           SRCNAME, __func__, m_mail, prop_name);
                   TRETURN S_OK;
                 }
             }
-          log_oom ("%s:%s: Message %p propchange: %ls.",
-                   SRCNAME, __func__, m_object, prop_name);
+          log_oom ("%s:%s: Mail %p propchange: %ls.",
+                   SRCNAME, __func__, m_mail, prop_name);
 
           if (!wcscmp (prop_name, L"SendUsingAccount"))
             {
@@ -297,6 +336,12 @@ EVENT_SINK_INVOKE(MailItemEvents)
               m_mail->setIsSendAgain (true);
               TRETURN S_OK;
             }
+          if (is_draft_mail (m_object))
+            {
+              log_oom ("%s:%s: Change allowed for draft",
+                       SRCNAME, __func__);
+              TRETURN S_OK;
+            }
 
           /* We have tried several scenarios to handle propery changes.
              Only save the property in MAPI and call MAPI SaveChanges
@@ -367,9 +412,16 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__, m_mail);
           if (!m_mail->needs_crypto_m () && m_mail->cryptState () == Mail::NoCryptMail)
             {
-             log_debug ("%s:%s: No crypto neccessary. Passing send for %p obj %p",
-                        SRCNAME, __func__, m_mail, m_object);
-             TBREAK;
+              if (m_mail->isCryptoMail ())
+                {
+                  log_debug ("%s:%s: Want to send crypto mail without crypto. "
+                             "Decrypting.", SRCNAME, __func__);
+                  m_mail->decryptPermanently_o ();
+                  TBREAK;
+                }
+              log_debug ("%s:%s: No crypto neccessary. Passing send for %p obj %p",
+                         SRCNAME, __func__, m_mail, m_object);
+              TBREAK;
             }
 
           if (parms->cArgs != 1 || parms->rgvarg[0].vt != (VT_BOOL | VT_BYREF))
@@ -385,24 +437,9 @@ EVENT_SINK_INVOKE(MailItemEvents)
               log_debug ("%s:%s: Send event for crypto mail %p saving and starting.",
                          SRCNAME, __func__, m_mail);
 
-              if (!m_mail->isAsyncCryptDisabled())
-                {
-                  /* Obtain a reference of the current item. This prevents
-                   * an early unload which would crash Outlook 2013
-                   *
-                   * As it didn't crash when the mail was opened in Outlook Spy this
-                   * mimics that the mail is inspected somewhere else. */
-                  m_mail->refCurrentItem ();
-                }
+              m_mail->prepareCrypto_o ();
+              m_mail->setIsDraftEncrypt (false);
 
-              // First contact with a mail to encrypt update
-              // state and oom data.
-              m_mail->updateOOMData_o ();
-
-              m_mail->setCryptState (Mail::NeedsFirstAfterWrite);
-
-              // Check inline response state before the write.
-              m_mail->check_inline_response ();
               // Save the Mail
               invoke_oom_method (m_object, "Save", NULL);
 
@@ -521,13 +558,15 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 {
                   log_error ("%s:%s: HrGetOneProp() failed: hr=%#lx\n",
                              SRCNAME, __func__, hr);
-                  gpgol_release (message);
                   TBREAK;
                 }
-              if (propval->Value.lpszA && !strstr (propval->Value.lpszA, "GpgOL"))
+              if (propval->Value.lpszA && !strstr (propval->Value.lpszA, "GpgOL") &&
+                  strcmp (propval->Value.lpszA, "IPM.Note.SMIME.MultipartSigned") &&
+                  strcmp (propval->Value.lpszA, "IPM.Note.SMIME"))
                 {
                   // Does not have a message class by us.
-                  log_debug ("%s:%s: Message %p - No GpgOL Message class after encryption. cls is: '%s'",
+                  log_debug ("%s:%s: Message %p - No GpgOL Message class "
+                             "after crypto. cls is: '%s'",
                              SRCNAME, __func__, m_object, propval->Value.lpszA);
                   log_debug ("%s:%s: Message %p - Activating T3656 Workaround",
                              SRCNAME, __func__, m_object);
@@ -593,8 +632,43 @@ EVENT_SINK_INVOKE(MailItemEvents)
               TBREAK;
             }
 
+          if (m_mail->cryptState() == Mail::NeedsFirstAfterWrite)
+            {
+              log_debug ("%s:%s: Crypto mail needs first after write. "
+                         "Auto-Passing but removing our categories.",
+                         SRCNAME, __func__);
+
+              m_mail->removeCategories_o ();
+              TBREAK;
+            }
+
           if (m_mail->isCryptoMail () && !m_mail->needsSave ())
             {
+              if (opt.draft_key && (m_mail->needs_crypto_m () & 1) &&
+                  is_draft_mail (m_object) && m_mail->decryptedSuccessfully ())
+                {
+                  if (m_mail->cryptState () == Mail::NeedsFirstAfterWrite ||
+                      m_mail->cryptState () == Mail::NeedsSecondAfterWrite)
+                    {
+                      log_debug ("%s:%s: re-encryption in progress. Passing.",
+                                 SRCNAME, __func__);
+                      TBREAK;
+                    }
+                  /* This is the case for a modified draft */
+                  log_debug ("%s:%s: Draft re-encryption starting now.",
+                             SRCNAME, __func__);
+                  m_mail->setIsDraftEncrypt (true);
+                  m_mail->prepareCrypto_o ();
+                  /* Passing write to trigger encrypt in after write */
+                  TBREAK;
+                }
+              if (m_mail->cryptState() == Mail::WantsSendMIME)
+                {
+                  log_debug ("%s:%s: Mail wants send mime. Passing.",
+                             SRCNAME, __func__);
+                  TBREAK;
+                }
+
               Mail *last_mail = Mail::getLastMail ();
               if (Mail::isValidPtr (last_mail))
                 {
@@ -656,9 +730,10 @@ EVENT_SINK_INVOKE(MailItemEvents)
               /* An error cleaning the mail should not happen normally.
                  But just in case there is an error we cancel the
                  write here. */
-              log_debug ("%s:%s: Failed to remove plaintext.",
+              log_debug ("%s:%s: Failed to remove plaintext. Canceling.",
                          SRCNAME, __func__);
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+              TRETURN S_OK;
             }
 
           if (!m_mail->isCryptoMail () && m_mail->is_forwarded_crypto_mail () &&
@@ -682,8 +757,29 @@ EVENT_SINK_INVOKE(MailItemEvents)
               m_mail->setIsForwardedCryptoMail (false);
             }
 
-          log_debug ("%s:%s: Passing write event.",
-                     SRCNAME, __func__);
+          if (m_mail->isDraftEncrypt () &&
+              m_mail->cryptState () != Mail::NeedsFirstAfterWrite &&
+              m_mail->cryptState () != Mail::NeedsSecondAfterWrite)
+            {
+              log_debug ("%s:%s: Canceling write because draft encrypt is in"
+                         " progress.",
+                         SRCNAME, __func__);
+              *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+              TRETURN S_OK;
+            }
+
+          if (opt.draft_key && (m_mail->needs_crypto_m () & 1) &&
+              !m_mail->isDraftEncrypt() &&
+              m_mail->cryptState() == Mail::NoCryptMail)
+            {
+              log_debug ("%s:%s: Draft encryption starting now.",
+                         SRCNAME, __func__);
+              m_mail->setIsDraftEncrypt (true);
+              m_mail->prepareCrypto_o ();
+            }
+
+          log_debug ("%s:%s: Passing write event. %i %i",
+                     SRCNAME, __func__, m_mail->isDraftEncrypt(), m_mail->cryptState());
           m_mail->setNeedsSave (false);
           TBREAK;
         }
@@ -700,6 +796,10 @@ EVENT_SINK_INVOKE(MailItemEvents)
                   log_debug ("%s:%s: Encrypt sign start failed.",
                              SRCNAME, __func__);
                   m_mail->setCryptState (Mail::NoCryptMail);
+                  if (!m_mail->isAsyncCryptDisabled())
+                    {
+                      m_mail->releaseCurrentItem();
+                    }
                 }
               TRETURN S_OK;
             }
@@ -707,6 +807,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
             {
               m_mail->setCryptState (Mail::NeedsUpdateInMAPI);
               m_mail->updateCryptMAPI_m ();
+              log_debug ("%s:%s: Second after write done.",
+                         SRCNAME, __func__);
               TRETURN S_OK;
             }
           TBREAK;
@@ -717,6 +819,15 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__, m_mail);
           if (m_mail->isCryptoMail ())
             {
+              if (is_draft_mail (m_object))
+                {
+                  /* In that case we want to ask the question to avoid data loss
+                    */
+                  log_oom ("%s:%s: Passing close because of draft status: %p",
+                           SRCNAME, __func__, m_mail);
+                  m_mail->setDecryptAgain (true);
+                  TBREAK;
+                }
               /* Close. This happens when an Opened mail is closed.
                  To prevent the question of wether or not to save the changes
                  (Which would save the decrypted data without an event to
@@ -761,11 +872,10 @@ EVENT_SINK_INVOKE(MailItemEvents)
           memdbg_dump ();
           TRETURN S_OK;
         }
-      /* Fallthrough */
       case ReplyAll:
       case Reply:
           is_reply = true;
-          __attribute__ ((fallthrough));
+          /* fall through */
       case Forward:
         {
           log_oom ("%s:%s: %s : %p",
