@@ -30,18 +30,17 @@
 extern "C" {
 #endif
 /* The filename of the attachment we create as the result of sign or
-   encrypt operations.  Don't change this name as some tests rely on
-   it.  */
-#define MIMEATTACHFILENAME "gpgolXXX.dat"
+   encrypt operations. */
+#define MIMEATTACHFILENAME "GpgOL_MIME_structure.txt"
+
 /* The name of the file we use to store the original body of PGP
    encrypted messages.  Note that PGP/MIME message don't need that
    because Outlook carries them as 2 attachments.  */
-#define PGPBODYFILENAME    "gpgolPGP.dat"
+#define PGPBODYFILENAME    "GpgOL_original_OpenPGP_message.txt"
 
 void log_mapi_property (LPMESSAGE message, ULONG prop, const char *propname);
 int get_gpgololdmsgclass_tag (LPMESSAGE message, ULONG *r_tag);
 int get_gpgolattachtype_tag (LPMESSAGE message, ULONG *r_tag);
-int get_gpgolsigstatus_tag (LPMESSAGE message, ULONG *r_tag);
 int get_gpgolprotectiv_tag (LPMESSAGE message, ULONG *r_tag);
 int get_gpgollastdecrypted_tag (LPMESSAGE message, ULONG *r_tag);
 int get_gpgolmimeinfo_tag (LPMESSAGE message, ULONG *r_tag);
@@ -82,11 +81,6 @@ LPSTREAM mapi_get_attach_as_stream (LPMESSAGE message,
 char *mapi_get_attach (LPMESSAGE message,
                        mapi_attach_item_t *item, size_t *r_nbytes);
 int mapi_mark_moss_attach (LPMESSAGE message, mapi_attach_item_t *item);
-int mapi_has_sig_status (LPMESSAGE msg);
-int mapi_test_sig_status (LPMESSAGE msg);
-char *mapi_get_sig_status (LPMESSAGE msg);
-
-int mapi_set_sig_status (LPMESSAGE message, const char *status_string);
 
 int mapi_set_gpgol_msg_class (LPMESSAGE message, const char *name);
 
@@ -116,16 +110,35 @@ int set_gpgol_draft_info_flags (LPMESSAGE message, int flags);
 /* Mark crypto attachments as hidden. And mark the moss
  attachment for later use. Returns the attachments position
  (1 is the first attachment) or 0 in case no attachment was found. */
-int mapi_mark_or_create_moss_attach (LPMESSAGE message, msgtype_t msgtype);
+int mapi_mark_or_create_moss_attach (LPMESSAGE base_message,
+                                     LPMESSAGE parsed_message,
+                                     msgtype_t msgtype);
 
 /* Copy the MAPI body to a PGPBODY type attachment. */
 int mapi_body_to_attachment (LPMESSAGE message);
 
 /* Get malloced uid of a message */
 char * mapi_get_uid (LPMESSAGE message);
+
+/* Remove the gpgol specific mapi tags */
+void mapi_delete_gpgol_tags (LPMESSAGE message);
+
+/* Explicitly change the message class */
+void mapi_set_mesage_class (LPMESSAGE message, const char *cls);
+
 #ifdef __cplusplus
 }
 #include <string>
+
+/* Parse the headers for additional useful fields.
+
+  r_autocrypt_info: Information about the autocrypt header.
+
+  A return value of false indicates an error. Check the
+  individual info fields "exists" values to check if
+  the header exists in the message */
+bool mapi_get_header_info (LPMESSAGE message,
+                           autocrypt_s &r_autocrypt_info);
 std::string mapi_get_header (LPMESSAGE message);
 #endif
 #endif /*MAPIHELP_H*/
